@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_AnalysisObject_h
 #define YODA_AnalysisObject_h
@@ -42,8 +42,7 @@ namespace YODA {
     /// Constructor giving a type, a path, another AO to copy annotation from, and an optional title
     AnalysisObject(const std::string& type, const std::string& path,
                    const AnalysisObject& ao, const std::string& title="") {
-      for (const std::string& a : ao.annotations())
-        setAnnotation(a, ao.annotation(a));
+      AnalysisObject::operator = (ao);
       setAnnotation("Type", type); // might override the copied ones
       setPath(path);
       setTitle(title);
@@ -59,9 +58,13 @@ namespace YODA {
     virtual ~AnalysisObject() { }
 
     /// Default copy assignment operator
-    virtual AnalysisObject& operator = (const AnalysisObject& ao) {
-      if (ao.path().length() > 0) setPath(ao.path());
-      if (ao.title().length() > 0) setTitle(ao.title());
+    virtual AnalysisObject& operator = (const AnalysisObject& ao) noexcept {
+      for (const std::string& a : ao.annotations()) {
+        if (a == "Type")  continue;
+        if (a == "Path"  && !ao.path().length())  continue;
+        if (a == "Title" && !ao.title().length())  continue;
+        setAnnotation(a, ao.annotation(a));
+      }
       return *this;
     }
 
@@ -78,8 +81,19 @@ namespace YODA {
     /// Reset this analysis object
     virtual void reset() = 0;
 
-    // variation parser
-    void parseVariations(){ return ; }
+    /// @}
+
+    /// @name I/O methods
+    /// @{
+
+    /// Render information about this analysis object
+    virtual void _renderYODA(std::ostream& os, const int width = 13) const noexcept = 0;
+
+    /// Render scatter-like information about this analysis object
+    virtual void _renderFLAT(std::ostream& os, const int width = 13) const noexcept = 0;
+
+    /// Return axis configuration (where applicable)
+    virtual std::string _config() const noexcept { return ""; }
 
     /// @}
 
@@ -139,53 +153,29 @@ namespace YODA {
     /// @note Templated on return type
     template <typename T>
     const T annotation(const std::string& name, const T& defaultreturn) const {
-      try {
-        std::string s = annotation(name);
-        return Utils::lexical_cast<T>(s);
-      } catch (const AnnotationError& ae) {
-        return defaultreturn;
-      }
+      Annotations::const_iterator v = _annotations.find(name);
+      if (v != _annotations.end()) return Utils::lexical_cast<T>(v->second);
+      return defaultreturn;
     }
 
 
-    /// @brief Add or set a string-valued annotation by name
-    void setAnnotation(const std::string& name, const std::string& value) {
-      _annotations[name] = value;
-    }
-
-    /// @brief Add or set a double-valued annotation by name
-    /// @todo Can we cover all FP types in one function via SFINAE?
-    void setAnnotation(const std::string& name, double value) {
-      // Recipe from Boost docs
-      std::stringstream ss;
-      ss << std::setprecision(std::numeric_limits<double>::max_digits10) << std::scientific << value;
-      setAnnotation(name, ss.str());
-    }
-
-    /// @brief Add or set a float-valued annotation by name
-    /// @todo Can we cover all FP types in one function via SFINAE?
-    void setAnnotation(const std::string& name, float value) {
-      // Recipe from Boost docs
-      std::stringstream ss;
-      ss << std::setprecision(std::numeric_limits<double>::max_digits10) << std::scientific << value;
-      setAnnotation(name, ss.str());
-    }
-
-    /// @brief Add or set a long-double-valued annotation by name
-    /// @todo Can we cover all FP types in one function via SFINAE?
-    void setAnnotation(const std::string& name, long double value) {
-      // Recipe from Boost docs
-      std::stringstream ss;
-      ss << std::setprecision(std::numeric_limits<double>::max_digits10) << std::scientific << value;
-      setAnnotation(name, ss.str());
-    }
-
     /// @brief Add or set an annotation by name (templated for remaining types)
     ///
     /// @note Templated on arg type, but stored as a string.
     template <typename T>
     void setAnnotation(const std::string& name, const T& value) {
-      setAnnotation(name, Utils::lexical_cast<std::string>(value));
+      if constexpr( std::is_floating_point<T>::value ) {
+        // Recipe from Boost docs
+        std::stringstream ss;
+        ss << std::setprecision(std::numeric_limits<double>::max_digits10) << std::scientific << value;
+        setAnnotation(name, ss.str());
+      }
+      else if constexpr( std::is_same<T, std::string>::value ) {
+        _annotations[name] = value;
+      }
+      else {
+        _annotations[name] = Utils::lexical_cast<std::string>(value);
+      }
     }
 
 
@@ -283,12 +273,80 @@ namespace YODA {
     ///
     /// @note For fillable types this is the dimension of the fill space (e.g. Histo1D -> dim=1).
     ///    For scatter types, it is the total dimension of the points (e.g. Scatter3D -> dim=3).
-    ///
-    /// @todo Provide a distinct fillDim() method on Fillables, and make dim always report the total dimension
-    virtual size_t dim() const = 0;
+    virtual size_t dim() const noexcept = 0;
+
+    /// @brief Return an inert version of the analysis object (e.g. scatter, estimate)
+    virtual AnalysisObject* mkInert(const std::string& path = "",
+                                    const std::string& source = "") const noexcept {
+      (void)source;
+      AnalysisObject* rtn = this->newclone();
+      rtn->setPath(path);
+      return rtn;
+    }
 
     /// @}
 
+    /// @name MPI serialisation
+    ///@{
+
+    /// @brief Length of serialized content vector for MPI reduce operations
+    virtual size_t lengthContent(bool fixed_length = false) const noexcept = 0;
+
+    /// @brief Content serialisation for MPI reduce operations
+    virtual std::vector<double> serializeContent(bool fixed_length = false) const noexcept = 0;
+
+    /// @brief Content deserialisation for MPI reduce operations
+    virtual void deserializeContent(const std::vector<double>& data) = 0;
+
+    /// @brief Length of serialized meta-data vector for MPI reduce operations
+    size_t lengthMeta(const bool skipPath = true,
+                      const bool skipTitle = true) const noexcept {
+      return 2*(_annotations.size() - skipPath - skipTitle);
+    }
+
+    /// @brief Mate-data serialisation for MPI reduce operations
+    std::vector<std::string> serializeMeta(const bool skipPath = true,
+                                           const bool skipTitle = true) const noexcept {
+
+      // Assemble annotations
+      std::vector<std::string> rtn;
+      rtn.reserve(2*(_annotations.size() - skipPath - skipTitle));
+      for (const auto& item : _annotations) {
+        if (item.first == "Type")  continue;
+        if (skipPath && item.first == "Path")  continue;
+        if (skipTitle && item.first == "Title")  continue;
+        rtn.push_back(item.first);
+        rtn.push_back(item.second);
+      }
+      return rtn;
+    }
+
+    /// @brief Mate-data deserialisation for MPI reduce operations
+    virtual void deserializeMeta(const std::vector<std::string>& data,
+                                 const bool resetPath = false, const bool resetTitle = false) {
+
+      if (data.empty())  return;
+      if (data.size() % 2)
+        throw  UserError("Expected even number of annotation elements (key-value pairs)!");
+
+      const std::string path = annotation("Path");
+      const std::string type = annotation("Type");
+      const std::string title = annotation("Title");
+      _annotations.clear();
+      _annotations["Type"] = type;
+      if (!resetPath)   _annotations["Path"] = path;
+      if (!resetTitle)  _annotations["Title"] = title;
+
+      auto itr = data.cbegin();
+      const auto itrEnd = data.cend();
+      while (itr != itrEnd) {
+        const std::string key = *itr; ++itr;
+        const std::string val = *itr; ++itr;
+        _annotations[key] = val;
+      }
+    }
+
+    /// @}
 
   private:
 
@@ -298,7 +356,7 @@ namespace YODA {
   };
 
 
-  // Convenience alias
+  /// Convenience alias
   using AO = AnalysisObject;
 
 
