@@ -1,14 +1,11 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2020 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_Fillable_h
 #define YODA_Fillable_h
 
-#include "YODA/AnalysisObject.h"
-#include "YODA/Exceptions.h"
-
 namespace YODA {
 
 
@@ -23,9 +20,7 @@ namespace YODA {
     virtual ~Fillable() = default;
 
 
-    /// Fill-dimension of this data object
-    virtual size_t fillDim() const = 0;
-
+    //@}
 
     /// @name Modifiers
     //@{
@@ -35,23 +30,26 @@ namespace YODA {
 
     /// Rescale as if all fill weights had been different by factor @a scalefactor
     virtual void scaleW(double scalefactor) = 0;
-    // setAnnotation("ScaledBy", annotation<double>("ScaledBy", 1.0) * scalefactor);
 
+    //@}
 
     /// @name Whole histo data
     //@{
 
+    /// Fill-dimension of this data object
+    virtual size_t fillDim() const noexcept = 0;
+
     /// Get the number of fills
-    virtual double numEntries(bool includeoverflows=true) const = 0;
+    virtual double numEntries(const bool includeoverflows=true) const = 0;
 
     /// Get the effective number of fills
-    virtual double effNumEntries(bool includeoverflows=true) const = 0;
+    virtual double effNumEntries(const bool includeoverflows=true) const = 0;
 
     /// Get sum of weights in histo
-    virtual double sumW(bool includeoverflows=true) const = 0;
+    virtual double sumW(const bool includeoverflows=true) const = 0;
 
     /// Get sum of squared weights in histo
-    virtual double sumW2(bool includeoverflows=true) const = 0;
+    virtual double sumW2(const bool includeoverflows=true) const = 0;
 
     //@}
 
