@@ -1,14 +1,13 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_IO_h
 #define YODA_IO_h
 
 #include "YODA/Writer.h"
 #include "YODA/Reader.h"
-#include "YODA/Index.h"
 
 namespace YODA {
 
@@ -83,9 +82,11 @@ namespace YODA {
   /// be determined from the filename.
   ///
   /// @todo Use SFINAE magic to allow ~arbitrary collection<AnalysisObject*> (with push_back()?) to be passed
-  inline void read(const std::string& filename, std::vector<AnalysisObject*>& aos) {
+  inline void read(const std::string& filename, std::vector<AnalysisObject*>& aos,
+                                                const std::string& match = "",
+                                                const std::string& unmatch = "") {
     Reader& r = mkReader(filename);
-    r.read(filename, aos);
+    r.read(filename, aos, match, unmatch);
   }
 
   /// @brief Read in a collection of objects from file @a filename.
@@ -94,18 +95,14 @@ namespace YODA {
   /// less CPU efficient than the alternative version where a vector is filled
   /// by reference. The appropriate format reader will be determined from the
   /// filename.
-  inline std::vector<AnalysisObject*> read(const std::string& filename) {
+  inline std::vector<AnalysisObject*> read(const std::string& filename,
+                                           const std::string& match = "",
+                                           const std::string& unmatch = "") {
     std::vector<AnalysisObject*> rtn;
-    read(filename, rtn);
+    read(filename, rtn, match, unmatch);
     return rtn;
   }
 
-  /// @brief Make index of a file.
-  inline Index mkIndex(const std::string& filename) {
-    Reader& r = mkReader(filename);
-    return r.mkIndex(filename);
-  }
-
   /// @}
 
 
@@ -118,9 +115,10 @@ namespace YODA {
   /// copying, and is hence CPU efficient.
   ///
   /// @todo Use SFINAE magic to allow ~arbitrary collection<AnalysisObject*> (with push_back()?) to be passed
-  inline void read(std::istream& is, std::vector<AnalysisObject*>& aos, const std::string& fmt) {
+  inline void read(std::istream& is, std::vector<AnalysisObject*>& aos, const std::string& fmt,
+                                     const std::string& match = "", const std::string& unmatch = "") {
     Reader& r = mkReader(fmt);
-    r.read(is, aos);
+    r.read(is, aos, match, unmatch);
   }
 
   /// @brief Read in a collection of objects from stream @a is, expecting format @a fmt.
@@ -128,9 +126,11 @@ namespace YODA {
   /// This version returns a vector by value, involving copying, and is hence
   /// less CPU efficient than the alternative version where a vector is filled
   /// by reference.
-  inline std::vector<AnalysisObject*> read(std::istream& is, const std::string& fmt) {
+  inline std::vector<AnalysisObject*> read(std::istream& is, const std::string& fmt,
+                                                             const std::string& match = "",
+                                                             const std::string& unmatch = "") {
     std::vector<AnalysisObject*> rtn;
-    read(is, rtn, fmt);
+    read(is, rtn, fmt, match, unmatch);
     return rtn;
   }
 
