@@ -1,31 +1,34 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2017 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_MathUtils_H
 #define YODA_MathUtils_H
 
 /// @todo Add SFINAE math type stuff (see Rivet) and add inrange() and inrange_closed_closed() etc. aliases cf. MCUtils
 
+#include "YODA/Exceptions.h"
 #include "YODA/Config/BuildConfig.h"
 
-#include <stdexcept>
-#include <string>
-#include <ostream>
-#include <sstream>
-#include <iostream>
-#include <cmath>
-#include <map>
-#include <vector>
-#include <utility>
 #include <algorithm>
 #include <functional>
 #include <numeric>
 #include <cassert>
-#include <limits>
-#include <climits>
 #include <cfloat>
+#include <climits>
+#include <cmath>
+#include <functional>
+#include <iostream>
+#include <limits>
+#include <map>
+#include <numeric>
+#include <ostream>
+#include <sstream>
+#include <stdexcept>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace YODA {
 
@@ -48,86 +51,91 @@ namespace YODA {
   enum Sign { MINUS = -1, ZERO = 0, PLUS = 1 };
 
 
-  /// @name Comparison functions for safe floating point equality tests
+  /// @name Comparison functions for safe (floating point) equality tests
   //@{
 
-  /// Compare a floating point number to zero with a degree
-  /// of fuzziness expressed by the absolute @a tolerance parameter.
-  inline bool isZero(double val, double tolerance=1E-8) {
-    return (fabs(val) < tolerance);
+  /// @brief Compare a number to zero
+  ///
+  /// This version for floating point types has a degree of fuzziness expressed
+  /// by the absolute @a tolerance parameter, for floating point safety.
+  template <typename NUM>
+  inline typename std::enable_if_t<std::is_floating_point_v<NUM>, bool>
+  isZero(NUM val, double tolerance=1e-8) {
+    return fabs(val) < tolerance;
   }
 
-  /// Compare an integral-type number to zero.
+  /// @brief Compare a number to zero
   ///
-  /// Since there is no risk of floating point error, this function just exists
-  /// in case @c isZero is accidentally used on an integer type, to avoid
-  /// implicit type conversion. The @a tolerance parameter is ignored.
-  inline bool isZero(long val, double=1E-8) {
-    return val == 0;
+  /// SFINAE template specialisation for integers, since there is no FP
+  /// precision issue.
+  template <typename NUM>
+  inline typename std::enable_if_t<std::is_integral_v<NUM>, bool>
+  isZero(NUM val, double = 1e-5) {
+    return val==0;
   }
 
+  /// @brief Check if a number is NaN
+  template <typename NUM>
+  inline typename std::enable_if_t<std::is_floating_point_v<NUM>, bool>
+  isNaN(NUM val) { return std::isnan(val); }
 
-  /// @brief Compare two floating point numbers for equality with a degree of fuzziness
+  /// @brief Check if a number is non-NaN
+  template <typename NUM>
+  inline typename std::enable_if_t<std::is_floating_point_v<NUM>, bool>
+  notNaN(NUM val) { return !std::isnan(val); }
+
+  /// @brief Compare two numbers for equality with a degree of fuzziness
   ///
-  /// The @a tolerance parameter is fractional, based on absolute values of the args.
-  inline bool fuzzyEquals(double a, double b, double tolerance=1E-5) {
-   const double absavg = (fabs(a) + fabs(b))/2.0;
-   const double absdiff = fabs(a - b);
-   const bool rtn = (isZero(a) && isZero(b)) || absdiff < tolerance*absavg;
-   // cout << a << " == " << b << "? " << rtn << endl;
-   return rtn;
+  /// This version for floating point types (if any argument is FP) has a degree
+  /// of fuzziness expressed by the fractional @a tolerance parameter, for
+  /// floating point safety.
+  template <typename N1, typename N2>
+  inline typename std::enable_if_t<
+    std::is_arithmetic_v<N1> && std::is_arithmetic_v<N2> &&
+   (std::is_floating_point_v<N1> || std::is_floating_point_v<N2>), bool>
+  fuzzyEquals(N1 a, N2 b, double tolerance=1e-5) {
+    const double absavg = (std::abs(a) + std::abs(b))/2.0;
+    const double absdiff = std::abs(a - b);
+    const bool rtn = (isZero(a) && isZero(b)) || absdiff < tolerance*absavg;
+    return rtn;
   }
 
-
-  /// @brief Compare two integral-type numbers for equality with a degree of fuzziness.
+  /// @brief Compare two numbers for equality with a degree of fuzziness
   ///
-  /// Since there is no risk of floating point error with integral types,
-  /// this function just exists in case @c fuzzyEquals is accidentally
-  /// used on an integer type, to avoid implicit type conversion. The @a
-  /// tolerance parameter is ignored, even if it would have an
-  /// absolute magnitude greater than 1.
-  inline bool fuzzyEquals(long a, long b, double=1E-5) {
+  /// Simpler SFINAE template specialisation for integers, since there is no FP
+  /// precision issue.
+  template <typename N1, typename N2>
+  inline typename std::enable_if_t<
+    std::is_integral_v<N1> && std::is_integral_v<N2>, bool>
+    fuzzyEquals(N1 a, N2 b, double) { //< NB. unused tolerance parameter for ints, still needs a default value!
     return a == b;
   }
 
+  /// @brief Comparator wrapper to use with STL algorithms, e.g. std::equal etc.
+  static std::function<bool(const double, const double)> fuzzyEqComp =
+    [](const double& lhs, const double& rhs) { return fuzzyEquals(lhs, rhs); };
 
-  /// @brief Compare two floating point numbers for >= with a degree of fuzziness
+
+  /// @brief Compare two numbers for >= with a degree of fuzziness
   ///
   /// The @a tolerance parameter on the equality test is as for @c fuzzyEquals.
-  inline bool fuzzyGtrEquals(double a, double b, double tolerance=1E-5) {
+  template <typename N1, typename N2>
+  inline typename std::enable_if_t<
+    std::is_arithmetic_v<N1> && std::is_arithmetic_v<N2>, bool>
+  fuzzyGtrEquals(N1 a, N2 b, double tolerance=1e-5) {
     return a > b || fuzzyEquals(a, b, tolerance);
   }
 
-  /// @brief Compare two integral-type numbers for >= with a degree of fuzziness.
-  ///
-  /// Since there is no risk of floating point error with integral types,
-  /// this function just exists in case @c fuzzyGtrEquals is accidentally
-  /// used on an integer type, to avoid implicit type conversion. The @a
-  /// tolerance parameter is ignored, even if it would have an
-  /// absolute magnitude greater than 1.
-  inline bool fuzzyGtrEquals(long a, long b, double=1E-5) {
-    return a >= b;
-  }
-
-
   /// @brief Compare two floating point numbers for <= with a degree of fuzziness
   ///
   /// The @a tolerance parameter on the equality test is as for @c fuzzyEquals.
-  inline bool fuzzyLessEquals(double a, double b, double tolerance=1E-5) {
+  template <typename N1, typename N2>
+  inline typename std::enable_if_t<
+    std::is_arithmetic_v<N1> && std::is_arithmetic_v<N2>, bool>
+  fuzzyLessEquals(N1 a, N2 b, double tolerance=1e-5) {
     return a < b || fuzzyEquals(a, b, tolerance);
   }
 
-  /// @brief Compare two integral-type numbers for <= with a degree of fuzziness.
-  ///
-  /// Since there is no risk of floating point error with integral types,
-  /// this function just exists in case @c fuzzyLessEquals is accidentally
-  /// used on an integer type, to avoid implicit type conversion. The @a
-  /// tolerance parameter is ignored, even if it would have an
-  /// absolute magnitude greater than 1.
-  inline bool fuzzyLessEquals(long a, long b, double=1E-5) {
-    return a <= b;
-  }
-
   /// Returns a number floored at the nth decimal place.
   inline double approx(double a, int n = 5) {
     double roundTo = pow(10.0,n);
@@ -251,8 +259,8 @@ namespace YODA {
   /// @note The arg ordering and the meaning of the nbins variable is "histogram-like",
   /// as opposed to the Numpy/Matlab version.
   inline std::vector<double> linspace(size_t nbins, double xmin, double xmax, bool include_end=true) {
-    assert(xmax >= xmin);
-    assert(nbins > 0);
+    if (xmax < xmin)  throw RangeError("xmax should not be smaller than xmin!");
+    if (nbins == 0)   throw RangeError("Requested number of bins is 0!");
     std::vector<double> rtn;
     const double interval = (xmax-xmin)/static_cast<double>(nbins);
     for (size_t i = 0; i < nbins; ++i) {
@@ -270,9 +278,9 @@ namespace YODA {
   /// as opposed to the Numpy/Matlab version, and the xmin and xmax arguments are expressed
   /// in "normal" space, rather than as the logarithms of the xmin/xmax values as in Numpy/Matlab.
   inline std::vector<double> logspace(size_t nbins, double xmin, double xmax, bool include_end=true) {
-    assert(xmax >= xmin);
-    assert(xmin > 0);
-    assert(nbins > 0);
+    if (xmax < xmin)  throw RangeError("xmax should not be smaller than xmin!");
+    if (xmin < 0)     throw RangeError("xmin should not be negative!");
+    if (nbins == 0)   throw RangeError("Requested number of bins is 0!");
     const double logxmin = std::log(xmin);
     const double logxmax = std::log(xmax);
     const std::vector<double> logvals = linspace(nbins, logxmin, logxmax);
@@ -363,7 +371,23 @@ namespace YODA {
   /// @name Statistics functions
   //@{
 
-  /// Calculate the mean of a sample
+  /// @brief Calculate the effective number of entries of a sample
+  inline double effNumEntries(const double sumW, const double sumW2) {
+    if (isZero(sumW2))  return 0;
+    return sqr(sumW) / sumW2;
+  }
+
+  /// @brief Calculate the effective number of entries of a sample
+  inline double effNumEntries(const std::vector<double>& weights) {
+    double sumW = 0.0, sumW2 = 0.0;
+    for (size_t i = 0; i < weights.size(); ++i) {
+      sumW += weights[i];
+      sumW2 += sqr(weights[i]);
+    }
+    return effNumEntries(sumW, sumW2);
+  }
+
+  /// @brief Calculate the mean of a sample
   inline double mean(const std::vector<int>& sample) {
     double mean = 0.0;
     for (size_t i=0; i<sample.size(); ++i) {
@@ -372,8 +396,119 @@ namespace YODA {
     return mean/sample.size();
   }
 
+  /// @brief Calculate the weighted mean of a sample
+  inline double mean(const double sumWX, const double sumW) {
+    return sumW? sumWX / sumW : std::numeric_limits<double>::quiet_NaN();
+  }
 
-  /// Calculate the covariance (variance) between two samples
+  /// @brief Calculate the weighted mean of a sample
+  inline double mean(const std::vector<double>& sample,
+                     const std::vector<double>& weights) {
+    if (sample.size() != weights.size())  throw RangeError("Inputs should have equal length!");
+    double sumWX = 0., sumW = 0.;
+    for (size_t i = 0; i < sample.size(); ++i) {
+      sumW  += weights[i];
+      sumWX += weights[i]*sample[i];
+    }
+    return mean(sumWX, sumW);
+  }
+
+  /// @brief Calculate the weighted variance of a sample
+  ///
+  /// Weighted variance defined as
+  /// sig2 = ( sum(wx**2) * sum(w) - sum(wx)**2 ) / ( sum(w)**2 - sum(w**2) )
+  /// see http://en.wikipedia.org/wiki/Weighted_mean
+  inline double variance(const double sumWX, const double sumW,
+                         const double sumWX2, const double sumW2) {
+    const double num = sumWX2*sumW - sqr(sumWX);
+    const double den = sqr(sumW) - sumW2;
+    /// @todo Isn't this sensitive to the overall scale of the weights?
+    /// Shouldn't it check if den is bigger then num by a set number of
+    /// orders of magnitude and vice versa?
+    // if (fabs(num) < 1e-10 && fabs(den) < 1e-10) {
+    //   return std::numeric_limits<double>::quiet_NaN();
+    // }
+    /// We take the modulus of the weighted variance
+    /// since the ratio can be negative with weighted means
+    /// @todo Is this the correct approach? There is no information
+    /// online other than "weights are non-negative"...
+    return den? fabs(num/den): std::numeric_limits<double>::quiet_NaN();
+  }
+
+  /// @brief Calculate the weighted variance of a sample
+  inline double variance(const std::vector<double>& sample,
+                         const std::vector<double>& weights) {
+    if (sample.size() != weights.size())  throw RangeError("Inputs should have equal length!");
+    if (fuzzyLessEquals(effNumEntries(weights), 1.0)) {
+       //throw LowStatsError("Requested variance of a distribution with only one effective entry");
+       return std::numeric_limits<double>::quiet_NaN();
+    }
+    double sumWX = 0., sumW = 0.;
+    double sumWX2 = 0., sumW2 = 0.;
+    for (size_t i = 0; i < sample.size(); ++i) {
+      sumW   += weights[i];
+      sumWX  += weights[i]*sample[i];
+      sumW2  += sqr(weights[i]);
+      sumWX2 += weights[i]*sqr(sample[i]);
+    }
+    return variance(sumWX, sumW, sumWX2, sumW2);
+  }
+
+  /// @brief Calculate the weighted standard deviation of a sample
+  inline double stdDev(const double sumWX, const double sumW,
+                       const double sumWX2, const double sumW2) {
+    return std::sqrt(variance(sumWX, sumW, sumWX2, sumW2));
+  }
+
+  /// @brief Calculate the weighted variance of a sample
+  inline double stdDev(const std::vector<double>& sample,
+                       const std::vector<double>& weights) {
+    return std::sqrt(variance(sample, weights));
+  }
+
+  /// @brief Calculate the weighted standard error of a sample
+  inline double stdErr(const double sumWX, const double sumW,
+                       const double sumWX2, const double sumW2) {
+    const double effN = effNumEntries(sumW, sumW2);
+    if (effN == 0)  return std::numeric_limits<double>::quiet_NaN();
+    const double var = variance(sumWX, sumW, sumWX2, sumW2);
+    return std::sqrt(var / effN);
+  }
+
+  /// @brief Calculate the weighted variance of a sample
+  inline double stdErr(const std::vector<double>& sample,
+                        const std::vector<double>& weights) {
+    if (sample.size() != weights.size())  throw RangeError("Inputs should have equal length!");
+    const double effN = effNumEntries(weights);
+    if (effN == 0)  return std::numeric_limits<double>::quiet_NaN();
+    const double var = variance(sample, weights);
+    return std::sqrt(var / effN);
+  }
+
+  /// @brief Calculate the weighted RMS of a sample
+  inline double RMS(const double sumWX2, const double sumW, const double sumW2) {
+    // Weighted RMS defined as
+    // rms = sqrt(sum{w x^2} / sum{w})
+    const double effN = effNumEntries(sumW, sumW2);
+    if (effN == 0)  return std::numeric_limits<double>::quiet_NaN();
+    const double meanSq = sumWX2 / sumW;
+    return std::sqrt(meanSq);
+  }
+
+  /// @brief Calculate the weighted RMS of a sample
+  inline double RMS(const std::vector<double>& sample,
+                    const std::vector<double>& weights) {
+    if (sample.size() != weights.size())  throw RangeError("Inputs should have equal length!");
+    double sumWX2 = 0., sumW = 0., sumW2 = 0.;
+    for (size_t i = 0; i < sample.size(); ++i) {
+      sumW   += weights[i];
+      sumW2  += sqr(weights[i]);
+      sumWX2 += weights[i]*sqr(sample[i]);
+    }
+    return RMS(sumWX2, sumW, sumW2);
+  }
+
+  /// @brief Calculate the covariance (variance) between two samples
   inline double covariance(const std::vector<int>& sample1, const std::vector<int>& sample2) {
     const double mean1 = mean(sample1);
     const double mean2 = mean(sample2);
@@ -388,7 +523,7 @@ namespace YODA {
   }
 
 
-  /// Calculate the correlation strength between two samples
+  /// @brief Calculate the correlation strength between two samples
   inline double correlation(const std::vector<int>& sample1, const std::vector<int>& sample2) {
     const double cov = covariance(sample1, sample2);
     const double var1 = covariance(sample1, sample1);
@@ -398,6 +533,50 @@ namespace YODA {
     return corr_strength;
   }
 
+
+  /// @brief Calculate the error-weighted chi2 statistic between two samples
+  ///
+  /// @note This calculation is rather naive as it neglects the error breakdowns that
+  /// may be available for the curves being compared. More sophisticated comparisons
+  /// will require a separate analysis based on the Estimate objects utilising the full
+  /// covariance information.
+  inline double naiveChi2(const std::vector<double>& sample1, const std::vector<double>& sample2,
+                     const std::vector<double>& s1errors = std::vector<double>{},
+                     const std::vector<double>& s2errors = std::vector<double>{}) {
+    if (sample1.size() != sample2.size()) {
+      throw RangeError("Inputs should have equal length!");
+    }
+    if (s1errors.size() && sample1.size() != s1errors.size()) {
+      throw RangeError("Inputs should have equal length!");
+    }
+    if (s2errors.size() && sample2.size() != s2errors.size()) {
+      throw RangeError("Inputs should have equal length!");
+    }
+    const size_t N = sample1.size();
+    double chi2 = 0.0;
+    for (size_t i = 0; i < N; ++i) {
+      double temp = sqr(sample1[i] - sample2[i]);
+      if (s1errors.size()) {
+        temp /= sqr(s1errors[i]) + sqr(s2errors[i]);
+      }
+      chi2 += temp;
+    }
+    return chi2;
+  }
+
+  /// @brief Calculate the error-weighted reduced chi2 statistic between two samples
+  ///
+  /// @note This calculation is rather naive as it neglects the error breakdowns that
+  /// may be available for the curves being compared. More sophisticated comparisons
+  /// will require a separate analysis based on the Estimate objects utilising the full
+  /// covariance information.
+  inline double naiveChi2reduced(const std::vector<double>& sample1, const std::vector<double>& sample2,
+                        const std::vector<double>& s1errors = std::vector<double>{},
+                        const std::vector<double>& s2errors = std::vector<double>{}) {
+    if (sample1.empty()) throw RangeError("Inputs should not have 0 length!");
+    return naiveChi2(sample1, sample2, s1errors, s2errors)/sample1.size();
+  }
+
   //@}
 
 
