@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2017 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_STRINGUTILS_H
 #define YODA_STRINGUTILS_H
@@ -9,8 +9,8 @@
 #include <vector>
 #include <sstream>
 #include <algorithm>
+#include <limits>
 #include <stdexcept>
-//#include <iostream>
 
 namespace YODA {
   namespace Utils {
@@ -45,6 +45,17 @@ namespace YODA {
     }
 
 
+    /// Convert a string to a double, accounting for out-of-range errors
+    inline double toDbl(const std::string& s) {
+      try {
+        return std::stod(s);
+      }
+      catch (...) {
+        return std::numeric_limits<double>::quiet_NaN();
+      }
+    }
+
+
     /// Convert a string to lower-case
     inline std::string toLower(const std::string& s) {
       std::string out = s;
@@ -133,8 +144,7 @@ namespace YODA {
       return itrim(s2);
     }
 
-
-    /// Split a string on a specified separator string
+    /// @brief Split a string on a specified separator string
     inline std::vector<std::string> split(const std::string& s, const std::string& sep) {
       std::vector<std::string> dirs;
       std::string tmp = s;
@@ -156,9 +166,9 @@ namespace YODA {
       return split(path, ":");
     }
 
-
   }
 
+
   // Annoyingly, this doesn't work without significant SFINAE etc., due to clashes with existing string & stream operators
   // /// string + any appending operator for convenience in the YODA namespace
   // template <typename T>
