@@ -1,25 +1,16 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_Writer_h
 #define YODA_Writer_h
 
 #include "YODA/AnalysisObject.h"
-#include "YODA/Counter.h"
-#include "YODA/Histo1D.h"
-#include "YODA/Histo2D.h"
-#include "YODA/Profile1D.h"
-#include "YODA/Profile2D.h"
-#include "YODA/Scatter1D.h"
-#include "YODA/Scatter2D.h"
-#include "YODA/Scatter3D.h"
 #include "YODA/Utils/Traits.h"
 #include <type_traits>
 #include <fstream>
-#include <ostream>
-#include <string>
+#include <iostream>
 
 namespace YODA {
 
@@ -47,14 +38,14 @@ namespace YODA {
 
     /// Write out pointer-like object @a ao to output stream @a stream.
     template <typename T>
-    typename std::enable_if<DerefableToAO<T>::value>::type //< -> void if valid
+    typename std::enable_if_t<DerefableToAO<T>::value> //< -> void if valid
     write(std::ostream& stream, const T& ao) {
       write(stream, *ao);
     }
 
     /// Write out pointer-like object @a ao to file @a filename.
     template <typename T>
-    typename std::enable_if<DerefableToAO<T>::value>::type //< -> void if valid
+    typename std::enable_if_t<DerefableToAO<T>::value> //< -> void if valid
     write(const std::string& filename, const T& ao) {
       write(filename, *ao);
     }
@@ -79,14 +70,14 @@ namespace YODA {
     /// @note The enable_if call checks whether RANGE is const_iterable, if yes the return
     ///       type is void. If not, this template will not be a candidate in the lookup
     template <typename RANGE>
-    typename std::enable_if<CIterable<RANGE>::value>::type
+    typename std::enable_if_t<CIterable<RANGE>::value>
     write(std::ostream& stream, const RANGE& aos) {
       write(stream, std::begin(aos), std::end(aos));
     }
 
     /// Write out a collection of objects @a objs to file @a filename.
     template <typename RANGE>
-    typename std::enable_if<CIterable<RANGE>::value>::type
+    typename std::enable_if_t<CIterable<RANGE>::value>
     write(const std::string& filename, const RANGE& aos) {
       write(filename, std::begin(aos), std::end(aos));
     }
@@ -148,19 +139,18 @@ namespace YODA {
 
     /// @}
 
-
     /// Set precision of numerical quantities in this writer's output.
     void setPrecision(int precision) {
       _precision = precision;
     }
 
     /// Set precision of numerical quantities for current AO in this writer's output.
-    void setAOPrecision(bool needsDP = false) {
+    void setAOPrecision(const bool needsDP = false) {
       _aoprecision = needsDP? std::numeric_limits<double>::max_digits10 : _precision;
     }
 
     /// Use libz compression?
-    void useCompression(bool compress=true) {
+    void useCompression(const bool compress=true) {
       _compress = compress;
     }
 
@@ -182,7 +172,7 @@ namespace YODA {
     /// @brief Write the body elements corresponding to AnalysisObject @a ao to @a stream
     /// @note Requires that @a ao is dereferenceable to an AnalysisObject, via the DerefableToAO<T> trait,
     template <typename T>
-    typename std::enable_if<DerefableToAO<T>::value>::type //< -> void if valid
+    typename std::enable_if_t<DerefableToAO<T>::value> //< -> void if valid
     writeBody(std::ostream& stream, const T& ao) { writeBody(stream, *ao); }
 
     /// Write any closing boilerplate required by the format to @a stream
@@ -194,14 +184,7 @@ namespace YODA {
     /// @name Specific AO type writer implementations, to be implemented in derived classes
     /// @{
 
-    virtual void writeCounter(std::ostream& stream, const Counter& c) = 0;
-    virtual void writeHisto1D(std::ostream& os, const Histo1D& h) = 0;
-    virtual void writeHisto2D(std::ostream& os, const Histo2D& h) = 0;
-    virtual void writeProfile1D(std::ostream& os, const Profile1D& p) = 0;
-    virtual void writeProfile2D(std::ostream& os, const Profile2D& p) = 0;
-    virtual void writeScatter1D(std::ostream& os, const Scatter1D& s) = 0;
-    virtual void writeScatter2D(std::ostream& os, const Scatter2D& s) = 0;
-    virtual void writeScatter3D(std::ostream& os, const Scatter3D& s) = 0;
+    virtual void writeAO(std::ostream& stream, const AnalysisObject& ao) = 0;
 
     /// @}
 
