@@ -30,5 +30,5 @@ def preprocess(s):
         if re.search(regex, s):
             s = s.replace(_from, _to)
 
-    return s
+    return s.replace(r'\newline', r'"+"\n"+r"')
 
