@@ -3,7 +3,7 @@
 from __future__ import print_function
 
 try:
-    from yoda.rootcompat import *
+    from yoda.rootcnv import *
 except ImportError as e:
     #print("YODA built without ROOT support")
     raise e
