@@ -1,35 +1,29 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
 #include "YODA/Counter.h"
 
 #include <cmath>
 #include <iostream>
+#include <iomanip>
+
 using namespace std;
 
 namespace YODA {
 
 
-  /// Copy constructor with optional new path
-  Counter::Counter(const Counter& c, const std::string& path)
-    : AnalysisObject("Counter", (path.size() == 0) ? c.path() : path, c, c.title())
-  {
-    _dbn = c._dbn;
-  }
-
-
   // Divide two counters
   /// @todo Add skipnullpts extra optional arg
-  Scatter1D divide(const Counter& numer, const Counter& denom) {
-    Scatter1D rtn;
+  Estimate0D divide(const Counter& numer, const Counter& denom) {
+    Estimate0D rtn;
+    if (numer.path() == denom.path())  rtn.setPath(numer.path());
+    if (rtn.hasAnnotation("ScaledBy")) rtn.rmAnnotation("ScaledBy");
     if (denom.val() != 0) {
       const double val = numer.val() / denom.val();
-      const double err = fabs(val) * add_quad(numer.relErr(), denom.relErr());
-      rtn.addPoint(val, err);
-    } else {
-      rtn.addPoint(std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN());
+      const double err = val * add_quad(numer.relErr(), denom.relErr());
+      rtn.set(val, {-err, err});
     }
     return rtn;
   }
@@ -37,11 +31,8 @@ namespace YODA {
 
   // Calculate a histogrammed efficiency ratio of two histograms
   /// @todo Add skipnullpts extra optional arg
-  Scatter1D efficiency(const Counter& accepted, const Counter& total) {
-    Scatter1D tmp = divide(accepted, total);
-    assert(tmp.numPoints() == 1);
-
-    /// BEGIN DIMENSIONALITY-INDEPENDENT BIT TO SHARE WITH H1
+  Estimate0D efficiency(const Counter& accepted, const Counter& total) {
+    Estimate0D tmp = divide(accepted, total);
 
     // Check that the numerator is consistent with being a subset of the denominator (NOT effNumEntries here!)
     if (accepted.numEntries() > total.numEntries() || accepted.sumW() > total.sumW())
@@ -57,11 +48,10 @@ namespace YODA {
       err = sqrt(abs( ((1-2*eff)*accepted.sumW2() + sqr(eff)*total.sumW2()) / sqr(total.sumW()) ));
     }
 
-    /// END DIMENSIONALITY-INDEPENDENT BIT TO SHARE WITH H1
-
-    tmp.point(0).setX(eff, err);
+    tmp.set(eff, {-err, err});
     return tmp;
   }
 
 
 }
+
