@@ -1,11 +1,9 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
-#include "YODA/Reader.h"
 #include "YODA/ReaderYODA.h"
-#include "YODA/ReaderAIDA.h"
 #include "YODA/ReaderFLAT.h"
 #include "YODA/Config/DummyConfig.h"
 
@@ -27,7 +25,6 @@ namespace YODA {
     }
     // Create the appropriate Reader
     if (Utils::startswith(fmt, "yoda")) return ReaderYODA::create();
-    if (Utils::startswith(fmt, "aida")) return ReaderAIDA::create();
     if (Utils::startswith(fmt, "dat" )) return ReaderFLAT::create(); ///< @todo Improve/remove... .ydat?
     if (Utils::startswith(fmt, "flat")) return ReaderFLAT::create();
     throw UserError("Format cannot be identified from string '" + name + "'");
