@@ -1,11 +1,9 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2023 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
 //
-#include "YODA/Writer.h"
 #include "YODA/WriterYODA.h"
-#include "YODA/WriterAIDA.h"
 #include "YODA/WriterFLAT.h"
 #include "YODA/Config/BuildConfig.h"
 
@@ -18,6 +16,7 @@
 #include <locale>
 #include <typeinfo>
 #include <sstream>
+#include <memory>
 using namespace std;
 
 namespace YODA {
@@ -39,7 +38,6 @@ namespace YODA {
     // Create the appropriate Writer
     Writer* w = nullptr;
     if (Utils::startswith(fmt, "yoda")) w = &WriterYODA::create();
-    if (Utils::startswith(fmt, "aida")) w = &WriterAIDA::create();
     if (Utils::startswith(fmt, "dat" )) w = &WriterFLAT::create(); ///< @todo Improve/remove... .ydat?
     if (Utils::startswith(fmt, "flat")) w = &WriterFLAT::create();
     if (!w) throw UserError("Format cannot be identified from string '" + name + "'");
@@ -69,7 +67,6 @@ namespace YODA {
       // zstr::ostream zstream(stream);
       // ostream& os = _compress ? zstream : stream;
       os = new zstr::ostream(stream);
-      //os = new zstr::ostream(stream, zstr::default_buff_size, Z_BEST_COMPRESSION);
       zos.reset(os);
       #else
       throw UserError("YODA was compiled without zlib support: can't write to a compressed stream");
@@ -104,29 +101,12 @@ namespace YODA {
   }
 
   void Writer::writeBody(ostream& stream, const AnalysisObject& ao) {
-    const string aotype = ao.type();
-    if (aotype == "Counter") {
-      writeCounter(stream, dynamic_cast<const Counter&>(ao));
-    } else if (aotype == "Histo1D") {
-      writeHisto1D(stream, dynamic_cast<const Histo1D&>(ao));
-    } else if (aotype == "Histo2D") {
-      writeHisto2D(stream, dynamic_cast<const Histo2D&>(ao));
-    } else if (aotype == "Profile1D") {
-      writeProfile1D(stream, dynamic_cast<const Profile1D&>(ao));
-    } else if (aotype == "Profile2D") {
-      writeProfile2D(stream, dynamic_cast<const Profile2D&>(ao));
-    } else if (aotype == "Scatter1D") {
-			writeScatter1D(stream, dynamic_cast<const Scatter1D&>(ao));
-    } else if (aotype == "Scatter2D") {
-      writeScatter2D(stream, dynamic_cast<const Scatter2D&>(ao));
-    } else if (aotype == "Scatter3D") {
-      writeScatter3D(stream, dynamic_cast<const Scatter3D&>(ao));
-    } else if (aotype[0] == '_') {
-      // Skip writing AO types with underscore prefixes (needed e.g. for Rivet wrappers)
-      // maybe write a comment line in the output
-    } else {
+    try {
+      writeAO(stream, ao);
+    }
+    catch(...) {
       ostringstream oss;
-      oss << "Unrecognised analysis object type " << aotype << " in Writer::write";
+      oss << "Unrecognised analysis object type " << ao.type() << " in Writer::write";
       throw Exception(oss.str());
     }
   }
