@@ -1,4 +1,4 @@
-#include "YODA/Histo2D.h"
+#include "YODA/Histo.h"
 #include "YODA/Utils/Formatting.h"
 
 #include <iostream>
