@@ -1,4 +1,4 @@
-#include "YODA/Profile1D.h"
+#include "YODA/Profile.h"
 #include "YODA/Utils/Formatting.h"
 
 using namespace YODA;
