@@ -1,4 +1,4 @@
-#include "YODA/Profile1D.h"
+#include "YODA/Profile.h"
 #include "YODA/Utils/Formatting.h"
 #include <cmath>
 #include <iostream>
@@ -18,11 +18,11 @@ int main() {
   }
 
   for (int i = 0; i < 4; ++i) {
-    if (i > 0) h.rebin(2);
+    if (i > 0) h.rebin<0>(2);
     MSG("Profile (rebinning #" << i << ", num bins = " << h.numBins() << ")");
-    for (const ProfileBin1D& b : h.bins()) {
+    for (const auto& b : h.bins()) {
       MSG(b.xMin() << "-" << b.xMax() << ": "
-          << RED(b.mean()) << ", " << BLUE(b.stdDev()) << ", " << RED(b.stdErr()));
+          << RED(b.mean(1)) << ", " << BLUE(b.xStdDev()) << ", " << RED(b.xStdErr()));
     }
   }
 
