@@ -1,4 +1,4 @@
-#include "YODA/Histo1D.h"
+#include "YODA/Histo.h"
 #include "YODA/WriterYODA.h"
 #include <cmath>
 #include <vector>
