@@ -11,14 +11,10 @@ aos = yoda.read(testfile, False, patterns=r".*_(eta|mass)_(2|4)")
 print(aos)
 assert len(aos) == 4
 
-index = yoda.mkIndexYODA(testfile)
-print("Index of ref file:")
-print(index)
-import re
-aos = yoda.read(testfile, False, patterns=[r".*_y_(1|3)", re.compile(r".*_dphi_(2|4)")])
+aos = yoda.read(testfile, False, patterns=[r".*_y_(1|3)", r".*_dphi_(2|4)"])
 print(aos)
 assert len(aos) == 3
 
-aos = yoda.read(testfile, False, patterns=[r".*_y_(1|3)", re.compile(r".*_dphi_(2|4)")], unpatterns=r".*_y_1")
+aos = yoda.read(testfile, False, patterns=[r".*_y_(1|3)", r".*_dphi_(2|4)"], unpatterns=r".*_y_1")
 print(aos)
 assert len(aos) == 2
