@@ -20,8 +20,8 @@ hs[2] += hs[1]
 print(hs[2])
 print()
 
-print(hs[0].annotations)
-print(hs[1].annotations)
+print(hs[0].annotations())
+print(hs[1].annotations())
 
 h = hs[0] + hs[1]
 print(hs[0])
