@@ -5,6 +5,7 @@ import yoda, random
 p = yoda.Profile2D(5,0.,10., 5,0.,10., "/bar")
 for _ in range(10000):
     p.fill(random.gauss(1, 3), random.gauss(1, 2), random.gauss(10, 0.5))
+    p.fill(random.gauss(1, 0.3), random.gauss(1, 0.2), random.gauss(1, 0.5))
 print(p)
 
 yoda.write([p], "p2d.yoda")
@@ -18,20 +19,18 @@ yoda.write([p], "p2d.dat")
 #     print ao
 
 ## Check scatter conversion
-s = yoda.mkScatter(p)
 s = p.mkScatter()
-s2 = s.mkScatter()
 
 if p.numBins() != s.numPoints():
     print("FAIL mkScatter() #bin={} -> #point={}".format(p.numBins(), s.numPoints()))
     exit(11)
-if p.yVals()[0] != s.point(0).y():
-    print("FAIL mkScatter() bin0 value={} -> point0 value={}".format(p.yVal(0), s.point(0).y()))
+idx = p.indexAt(1,1)
+zVal = p.bin(idx).zMean()
+if zVal != s.point(0).z():
+    print("FAIL mkScatter() bin0 value={} -> point0 value={}".format(zVal, s.point(0).z()))
     exit(12)
-if p.bin(0).stdErr() != s.point(0).zErrAvg():
-    print("FAIL mkScatter(p_usestddev=False) bin0 err={} -> point0 err={}".format(p.bin(0).stdErr(), s.point(0).zErrAvg()))
+zErr = p.bin(idx).zStdErr()
+if zErr != s.point(0).zErrAvg():
+    print("FAIL mkScatter() bin0 err={} -> point0 err={}".format(zErr, s.point(0).zErrAvg()))
     exit(13)
-s = yoda.mkScatter(p, p_usestddev=True)
-if p.bin(0).stdDev() != s.point(0).zErrAvg():
-    print("FAIL mkScatter(p_usestddev=True) bin0 err={} -> point0 err={}".format(p.bin(0).stdDev(), s.point(0).zErrAvg()))
-    exit(21)
+
